# Plugin for TWiki Enterprise Collaboration Platform, http://TWiki.org/
#
# Copyright (C) 2012 Hypertek lnc
# Copyright (C) 2012-2016 Peter Thoeny, peter[at]thoeny.org
# Copyright (C) 2012-2016 TWiki Contributors.
# All Rights Reserved. TWiki Contributors are listed in the AUTHORS
# file in the root of this distribution.
# NOTE: Please extend that file, not this notice.
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 3
# of the License, or (at your option) any later version. For
# more details read LICENSE in the root of this distribution.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
#
# For licensing info read LICENSE file in the TWiki root.

package TWiki::Plugins::SendMailPlugin;
use strict;
our $pluginName = 'SendMailPlugin';

require TWiki::Func;    # The plugins API
require TWiki::Plugins; # For the API version

our $VERSION = '$Rev: 29910 (2016-01-20) $';
our $RELEASE = '2016-01-19';

# One line description, is shown in the %SYSTEMWEB%.TextFormattingRules topic:
our $SHORTDESCRIPTION = 'Send e-mail from actions in TWiki topics, useful for workflow automation';

our $NO_PREFS_IN_TOPIC = 1;
our $debug;
our $web;
our $topic;

#=====================================================================
sub initPlugin {
    ( $topic, $web ) = @_;

    # check for Plugins.pm versions
    if( $TWiki::Plugins::VERSION < 1.1 ) {
        TWiki::Func::writeWarning( "Version mismatch between $pluginName and Plugins.pm" );
        return 0;
    }

    $debug = $TWiki::cfg{Plugins}{SendMailPlugin}{Debug} || 0;

    TWiki::Func::registerTagHandler( 'SENDMAIL', \&_SENDMAIL );
    TWiki::Func::registerRESTHandler( 'sendmail', \&_restSendmail );

    # Plugin correctly initialized
    return 1;
}

#=====================================================================
sub _restSendmail {
    my ($session) = @_;

    my $query = TWiki::Func::getCgiQuery();
    return "ERROR: Internal error, query object is missing" unless( $query );

    my $params;
    $params->{action} = $query->param( 'action' );
    return "ERROR: Parameter 'action=send' is missing"
      unless( $params->{action} && $params->{action} eq 'send' );

    $params->{excludetopic} = $query->param( 'excludetopic' );
    return "ERROR: No action, topic matches parameter 'excludetopic'"
      if( $params->{excludetopic} && $params->{excludetopic} =~ /^(on|$topic|$web\.$topic)$/ );

    $params->{from}      = $query->param( 'from' );
    $params->{to}        = $query->param( 'to' );
    $params->{cc}        = $query->param( 'cc' );
    $params->{bcc}       = $query->param( 'bcc' );
    $params->{subject}   = $query->param( 'subject' );
    $params->{plaintext} = $query->param( 'plaintext' );
    $params->{text}      = $query->param( 'text' );
    $params->{htmltext}  = $query->param( 'htmltext' );
    $params->{onsuccess} = $query->param( 'onsuccess' );
    $params->{onerror}   = $query->param( 'onerror' ) || 'ERROR: Send error, $error';

    return _SENDMAIL( $session, $params, $topic, $web );
}

#=====================================================================
sub _SENDMAIL {
    my( $session, $params, $theTopic, $theWeb ) = @_;

    my $action = $params->{action};
    return '' unless( $action && $action eq 'send' );

    return '' if( $params->{excludetopic} &&
                  $params->{excludetopic} =~ /^(on|$theTopic|$theWeb\.$theTopic)$/ );

    my $from      = expandEmail( $TWiki::cfg{Plugins}{SendMailPlugin}{From}
                    || $params->{from} || '$webmastername <$webmasteremail>' );
    my $to        = expandEmail( $TWiki::cfg{Plugins}{SendMailPlugin}{To}
                    || $params->{to}   || '$webmastername <$webmasteremail>' );;
    my $cc        = expandEmail( $TWiki::cfg{Plugins}{SendMailPlugin}{CC}
                    || $params->{cc}   || '' );
    my $bcc       = expandEmail( $TWiki::cfg{Plugins}{SendMailPlugin}{BCC}
                    || $params->{bcc}  || '' );
    my $subject   = TWiki::Func::decodeFormatTokens( $params->{subject}
                    || 'SendMailPlugin Note: For subject specify subject="..." parameter' );
    my $plain     = TWiki::Func::decodeFormatTokens( $params->{plaintext} || $params->{text} || '' );
    my $html      = TWiki::Func::decodeFormatTokens( $params->{htmltext}  || '' );
    my $onSuccess = TWiki::Func::decodeFormatTokens( $params->{onsuccess} || '' );
    my $onError   = TWiki::Func::decodeFormatTokens( $params->{onerror}   || '$error' );

    my $email = "From: $from\n";
    $email   .= "To: $to\n";
    $email   .= "CC: $cc\n"   if( $cc  && $cc  !~ /^disable$/i );
    $email   .= "BCC: $bcc\n" if( $bcc && $bcc !~ /^disable$/i );
    $email   .= "Subject: $subject\n";
    if( $plain && $html ) {
        # MIME multi-part e-mail
        $email .=
            "MIME-Version: 1.0\n"
          . "Content-Type: multipart/alternative; boundary=\"_=_TWiki_MuIti_Part_B0undary\"\n"
          . "\n"
          . "This is a multi-part message in MIME format.\n"
          . "--_=_TWiki_MuIti_Part_B0undary\n"
          . "Content-Type: text/plain; charset=iso-8859-1; format=flowed\n"
          . "Content-Transfer-Encoding: 8bit\n"
          . "\n"
          . "$plain\n"
          . "--_=_TWiki_MuIti_Part_B0undary\n"
          . "Content-Type: text/html; charset=iso-8859-1\n"
          . "Content-Transfer-Encoding: 8bit\n"
          . "\n"
          . "$html\n"
          . "--_=_TWiki_MuIti_Part_B0undary--\n";
    } elsif( $html ) {
        # html only e-mail
        $email .=
            "Content-Type: text/html; charset=iso-8859-1\n"
          . "Content-Transfer-Encoding: 8bit\n"
          . "\n"
          . "$html\n";
    } elsif( $plain ) {
        # plain text e-mail
        $email .=
            "Content-Type: text/plain; charset=iso-8859-1; format=flowed\n"
          . "Content-Transfer-Encoding: 8bit\n"
          . "\n"
          . "$plain\n";
    } else {
        # help message in plain text
        $email .=
            "Content-Type: text/plain; charset=iso-8859-1; format=flowed\n"
          . "Content-Transfer-Encoding: 8bit\n"
          . "\n"
          . 'SendMailPlugin Note: For e-mail body specify a text="..." parameter, '
          . 'and possibly a htmltext="..." parameter with HTML message.' . "\n";
    }
    if( $debug ) {
        TWiki::Func::writeDebug( "TWiki::Plugins::SendMailPlugin e-mail:" );
        TWiki::Func::writeDebug( "===( START )=============" );
        TWiki::Func::writeDebug( "$email" );
        TWiki::Func::writeDebug( "===(  END  )=============" );
    }
    my $sendErr = TWiki::Func::sendEmail( $email );
    if( $sendErr ) {
        $sendErr =~ s/[\n\r]/ /go;
        if( $debug ) {
            TWiki::Func::writeDebug( "TWiki::Plugins::SendMailPlugin e-mail error: $sendErr" );
        }
        $onError =~ s/\$error/$sendErr/g;
        return $onError;
    }
    return $onSuccess;
}

#=====================================================================
sub expandEmail {
    my( $text ) = @_;

    return '' unless( $text );
    my $userEmail = join( ', ', TWiki::Func::wikinameToEmails() );
    my $userWikiName = TWiki::Func::getWikiName();
    my @emails = ();
    foreach my $item ( split( /, */, $text ) ) {
        if( TWiki::Func::isValidWikiWord( $item ) ) {
            foreach my $mail ( TWiki::Func::wikinameToEmails( $item ) ) {
                push( @emails, "$item <$mail>" );
            }
        } else {
            $item =~ s/\$useremail/$userEmail/go;
            $item =~ s/\$username/$userWikiName/go;
            $item =~ s/\$webmasteremail/$TWiki::cfg{WebMasterEmail}/go;
            $item =~ s/\$webmastername/$TWiki::cfg{WebMasterName}/go;
            push( @emails, $item );
        }
    }
    $text = join( ', ', @emails );
    return $text;
}

#=====================================================================
1;
